import { reactive } from 'vue';

import i18n from '@/i18n';
import { parseColor } from '@/hook/color';

const { t } = i18n.global;

export const colorManager = reactive({
    currentGerneralIndex: 0,
    generals: [
        {
            value: 0,
            type: 'wire',
            label: t('common.line'),
            color: 'white',
        },
        {
            value: 1,
            type: 'port',
            label: t('common.port'),
            color: 'white',
        },
        {
            value: 2,
            type: 'instance',
            label: t('common.instance'),
            color: 'white',
        },
        {
            value: 3,
            type: 'cross-dot',
            label: t('cross-dot'),
            color: 'white'
        },
        {
            value: 4,
            type: 'wire-active',
            label: t('common.selected-wire'),
            color: 'white'
        }
    ],

    currentCell: 'binary',
    cells: [
        {
            value: '',
            text: ''
        },
        {

        },
        {

        },
        {

        },
        {

        }
    ],

    initColor() {
        const rootStyles = getComputedStyle(document.documentElement);
        for (const item of this.generals) {
            const optionName = `--${item.type}-color`;
            const colorString = rootStyles.getPropertyValue(optionName);
            item.color = colorString;
        }
    }
});

/**
 * @description 通用颜色设置发生变时
 * @param {string} colorString 用户选择的颜色的字符串 
 */
export function onGeneralColorChange(colorString) {
    const { r, g, b } = parseColor(colorString);
    const index = colorManager.currentGerneralIndex;
    const item = colorManager.generals[index];
    const borderColor = `rgb(${r}, ${g}, ${b})`;
    const fillColor = `rgba(${r}, ${g}, ${b}, 0.1)`;
    const rootStyles = getComputedStyle(document.documentElement);
    document.documentElement.style.setProperty(`--${item.type}-color`, borderColor);
    document.documentElement.style.setProperty(`--${item.type}-fill-color`, fillColor);
}

export const predefinedColors = [
    '#ff4500',
    '#ff8c00',
    '#ffd700',
    '#90ee90',
    '#00ced1',
    '#1e90ff',
    '#c71585'
];